import { AssertionUnitConfigBuilder } from "./AssertionUnitConfigBuilder";
import { AssignmentUnitConfigBuilder } from "./AssignmentUnitConfigBuilder";
import { ExecutionUnitConfigBuilder } from "./ExecutionUnitConfigBuilder";
import { IOUnitConfigBuilder } from "./IOUnitConfigBuilder";
import { UnitConfigBuilder } from "./UnitConfigBuilder";

const builders = {
    UnitConfigBuilder,
    AssignmentUnitConfigBuilder,
    AssertionUnitConfigBuilder,
    ExecutionUnitConfigBuilder,
    IOUnitConfigBuilder,
};

export { builders };
